@@ -222,7 +222,7 @@ def pendulum_to_utc_datetime_without_tz(x: DateTime) -> datetime.datetime:
         dt_utc_from_moscow = pendulum_to_utc_datetime_without_tz(in_moscow)  # 6am, no timezone info
         dt_utc_from_london = pendulum_to_utc_datetime_without_tz(in_london)  # 6am, no timezone info
 
-    """  # noqa
+    """  # noqa: E501
     pendulum_in_utc = pendulum.UTC.convert(x)
     return pendulum_to_datetime_stripping_tz(pendulum_in_utc)
 
@@ -307,7 +307,7 @@ def strfdelta(
 
     Modified from
     https://stackoverflow.com/questions/538666/python-format-timedelta-to-string
-    """  # noqa
+    """
 
     # Convert tdelta to integer seconds.
     if inputtype == "timedelta":
@@ -639,7 +639,7 @@ def duration_to_iso(
       realistic (negative, 1000 years, 11 months, and the maximum length for
       seconds/microseconds).
 
-    """  # noqa
+    """
     prefix = ""
     negative = d < Duration()
     if negative and minus_sign_at_front:
