@@ -28,10 +28,15 @@
 
 from contextlib import contextmanager
 
-from sqlalchemy.orm import Session as SqlASession
+from sqlalchemy.engine.base import Engine
+from sqlalchemy.orm.session import Session as SqlASession
+from sqlalchemy.schema import DDL
 
-from cardinal_pythonlib.sqlalchemy.dialect import quote_identifier
-from cardinal_pythonlib.sqlalchemy.engine_func import is_sqlserver
+from cardinal_pythonlib.sqlalchemy.dialect import (
+    quote_identifier,
+    SqlaDialectName,
+)
+from cardinal_pythonlib.sqlalchemy.schema import execute_ddl
 from cardinal_pythonlib.sqlalchemy.session import get_engine_from_session
 
 
@@ -40,6 +45,14 @@ from cardinal_pythonlib.sqlalchemy.session import get_engine_from_session
 # =============================================================================
 
 
+def _exec_ddl_if_sqlserver(engine: Engine, sql: str) -> None:
+    """
+    Execute DDL only if we are running on Microsoft SQL Server.
+    """
+    ddl = DDL(sql).execute_if(dialect=SqlaDialectName.SQLSERVER)
+    execute_ddl(engine, ddl=ddl)
+
+
 @contextmanager
 def if_sqlserver_disable_constraints(
     session: SqlASession, tablename: str
@@ -54,19 +67,18 @@ def if_sqlserver_disable_constraints(
 
     See
     https://stackoverflow.com/questions/123558/sql-server-2005-t-sql-to-temporarily-disable-a-trigger
-    """  # noqa
+    """
     engine = get_engine_from_session(session)
-    if is_sqlserver(engine):
-        quoted_tablename = quote_identifier(tablename, engine)
-        session.execute(
-            f"ALTER TABLE {quoted_tablename} NOCHECK CONSTRAINT all"
-        )
-        yield
-        session.execute(
-            f"ALTER TABLE {quoted_tablename} WITH CHECK CHECK CONSTRAINT all"
-        )
-    else:
-        yield
+    quoted_tablename = quote_identifier(tablename, engine)
+    _exec_ddl_if_sqlserver(
+        engine, f"ALTER TABLE {quoted_tablename} NOCHECK CONSTRAINT all"
+    )
+    yield
+    _exec_ddl_if_sqlserver(
+        engine,
+        f"ALTER TABLE {quoted_tablename} WITH CHECK CHECK CONSTRAINT all",
+    )
+    # "CHECK CHECK" is correct here.
 
 
 @contextmanager
@@ -83,15 +95,16 @@ def if_sqlserver_disable_triggers(
 
     See
     https://stackoverflow.com/questions/123558/sql-server-2005-t-sql-to-temporarily-disable-a-trigger
-    """  # noqa
+    """
     engine = get_engine_from_session(session)
-    if is_sqlserver(engine):
-        quoted_tablename = quote_identifier(tablename, engine)
-        session.execute(f"ALTER TABLE {quoted_tablename} DISABLE TRIGGER all")
-        yield
-        session.execute(f"ALTER TABLE {quoted_tablename} ENABLE TRIGGER all")
-    else:
-        yield
+    quoted_tablename = quote_identifier(tablename, engine)
+    _exec_ddl_if_sqlserver(
+        engine, f"ALTER TABLE {quoted_tablename} DISABLE TRIGGER all"
+    )
+    yield
+    _exec_ddl_if_sqlserver(
+        engine, f"ALTER TABLE {quoted_tablename} ENABLE TRIGGER all"
+    )
 
 
 @contextmanager
